@@ -27,12 +27,25 @@
 #include "gps-data-logger-widget.h"
 
 #define PADDING 10
+#define THEME_DIR "/etc/hildon/theme/images/"
+#define BUTTON_BG THEME_DIR "ApplicationShortcutApplet.png"
+#define BUTTON_PRESSED_BG THEME_DIR "ApplicationShortcutAppletPressed.png"
+#define REC_ICON "camera_video_recording"
+#define STOP_ICON "camera_video_stop"
+#define PAUSE_ICON "camera_video_pause"
+#define GPS_REFRESH_ICON "general_refresh"
+#define GPS_ON_ICON "gps_location"
+#define GPS_OFF_ICON "gps_not_connected"
 
 HD_DEFINE_PLUGIN_MODULE (GpsDataLoggerWidget, gps_data_logger_widget, HD_TYPE_HOME_PLUGIN_ITEM)
 
 static void
 update_status_and_buttons (GpsDataLoggerWidget * self);
 
+/* -------------------------------------------------------------------- */
+/* Settings
+ */
+
 static void
 apply_settings (GpsDataLoggerWidget * self)
 {
@@ -270,10 +283,8 @@ settings_dialog_new (GpsDataLoggerWidget * self)
     return GTK_DIALOG (dialog);
 }
 
-
-
 void
-settings_dialog_show (GtkWidget *widget,
+on_settings_dialog_show (GtkWidget *widget,
 			 gpointer data)
 {
     GpsDataLoggerWidget *self = GPS_DATA_LOGGER_WIDGET (data);
@@ -285,12 +296,108 @@ settings_dialog_show (GtkWidget *widget,
     } else {
 	gtk_widget_show (self->settings_dialog);
     }
-    /*
-    gtk_dialog_run (GTK_DIALOG (self->settings_dialog));
-    gtk_widget_destroy (self->settings_dialog);
-    */
 }
 
+/* -------------------------------------------------------------------- */
+/* Pixbufs and icons
+ */
+
+static void
+free_pixbuf(GdkPixbuf **pixbuf)
+{
+    if (*pixbuf)
+    {
+        g_object_unref(*pixbuf);
+        *pixbuf = NULL;
+    }
+}
+
+static void
+free_theme_pixbufs(GpsDataLoggerWidget * self)
+{
+    free_pixbuf(&self->button_bg);
+    free_pixbuf(&self->button_pressed_bg);
+}
+
+static void
+load_theme_pixbufs(GpsDataLoggerWidget * self)
+{
+    self->button_bg = gdk_pixbuf_new_from_file(BUTTON_BG, NULL);
+    self->button_pressed_bg = gdk_pixbuf_new_from_file(BUTTON_PRESSED_BG, NULL);
+}
+
+static gboolean
+theme_pixbufs_loaded(GpsDataLoggerWidget * self)
+{
+    if (self->button_bg && self->button_pressed_bg)
+        return TRUE;
+
+    return FALSE;
+}
+
+static gboolean
+on_style_set(GtkWidget *widget, GtkStyle *previous_style, gpointer data)
+{
+    GpsDataLoggerWidget *self = GPS_DATA_LOGGER_WIDGET (data);
+
+    /* if pixmaps already loaded, free them */
+    if (theme_pixbufs_loaded(self))
+            free_theme_pixbufs(self);
+
+    load_theme_pixbufs(self);
+
+    if (!theme_pixbufs_loaded(self))
+            free_theme_pixbufs(self);
+
+    return FALSE;
+}
+
+static void
+free_icons(GpsDataLoggerWidget * self)
+{
+        free_pixbuf(&self->rec_icon);
+        free_pixbuf(&self->stop_icon);
+        free_pixbuf(&self->pause_icon);
+	free_pixbuf(&self->gps_refresh_icon);
+	free_pixbuf(&self->gps_on_icon);
+	free_pixbuf(&self->gps_off_icon);
+}
+
+static void
+load_icons(GpsDataLoggerWidget * self)
+{
+        GtkIconTheme *theme;
+
+        theme = gtk_icon_theme_get_default();
+
+        /* Load icons */
+        self->rec_icon = gtk_icon_theme_load_icon(theme,
+                        REC_ICON, 48, 0, NULL);
+        self->stop_icon = gtk_icon_theme_load_icon(theme,
+                        STOP_ICON, 48, 0, NULL);
+        self->pause_icon = gtk_icon_theme_load_icon(theme,
+                        PAUSE_ICON, 48, 0, NULL);
+        self->gps_refresh_icon = gtk_icon_theme_load_icon(theme,
+                        GPS_REFRESH_ICON, 48, 0, NULL);
+        self->gps_on_icon = gtk_icon_theme_load_icon(theme,
+                        GPS_ON_ICON, 48, 0, NULL);
+        self->gps_off_icon = gtk_icon_theme_load_icon(theme,
+                        GPS_OFF_ICON, 48, 0, NULL);
+}
+
+/* -------------------------------------------------------------------- */
+
+static gboolean
+on_widget_leave(GtkWidget* widget, GdkEventCrossing *event, gpointer data)
+{
+    GpsDataLoggerWidget *self = GPS_DATA_LOGGER_WIDGET (data);
+
+    self->rec_button_down = FALSE;
+    self->stop_button_down = FALSE;
+    gtk_widget_queue_draw(widget);
+
+    return FALSE;
+}
 
 static void
 logfile_close(GpsDataLoggerWidget * self)
@@ -340,6 +447,9 @@ logfile_start(GpsDataLoggerWidget * self)
                         NULL,
                         "Could not open \"%s\" for writing!",
 			path);
+#ifndef __arm__
+	g_print("Could not open logfile for writing, going to IDLE\n");
+#endif
         self->app_state = APP_STATE_IDLE;
         location_gpsd_control_stop(self->control);
         update_status_and_buttons(self);
@@ -359,77 +469,49 @@ logfile_start(GpsDataLoggerWidget * self)
 static void
 update_status_and_buttons (GpsDataLoggerWidget * self)
 {
-    GtkIconTheme *icon_theme;
-    GdkPixbuf *status_icon, *button_rec_icon;
-    GtkWidget *button_rec_image;
-    gboolean update_status_icon = FALSE;
-    gboolean update_button_image = FALSE;
-
-    icon_theme = gtk_icon_theme_get_default ();
     switch(self->app_state) {
 	case APP_STATE_STARTING:
-		status_icon = gtk_icon_theme_load_icon (icon_theme, "general_refresh", 48, 0, NULL);
-		gtk_widget_set_sensitive(self->button_rec, FALSE);
-		gtk_widget_set_sensitive(self->button_stop, TRUE);
-		update_status_icon = TRUE;
+		gtk_image_set_from_pixbuf (GTK_IMAGE(self->gps_status), 
+				           self->gps_refresh_icon);
+		self->rec_button_enabled = FALSE;
+		self->stop_button_enabled = TRUE;
 #ifndef __arm__
 		g_print(" starting: loading new icon, disabling rec\n");
 #endif
 		break;
 	case APP_STATE_RECORDING:
-		status_icon = gtk_icon_theme_load_icon (icon_theme, "gps_location", 48, 0, NULL);
-		button_rec_icon = gtk_icon_theme_load_icon (icon_theme, "camera_video_pause", 48, 0, NULL);
-		update_status_icon = TRUE;
-		update_button_image = TRUE;
-		gtk_widget_set_sensitive(self->button_rec, TRUE);
-		gtk_widget_set_sensitive(self->button_stop, TRUE);
+		gtk_image_set_from_pixbuf (GTK_IMAGE(self->gps_status), 
+				           self->gps_on_icon);
+		self->pause_button_visible = TRUE;
+		self->rec_button_enabled = TRUE;
+		self->stop_button_enabled = TRUE;
 		break;
 	case APP_STATE_PAUSED:
-		button_rec_icon = gtk_icon_theme_load_icon (icon_theme, "camera_video_recording", 48, 0, NULL);
-		update_button_image = TRUE;
+		self->pause_button_visible = FALSE;
 		break;
 	case APP_STATE_STOPPING:
-		status_icon = gtk_icon_theme_load_icon (icon_theme, "gps_not_connected", 48, 0, NULL);
-		button_rec_icon = gtk_icon_theme_load_icon (icon_theme, "camera_video_recording", 48, 0, NULL);
-		update_status_icon = TRUE;
-		update_button_image = TRUE;
-		gtk_widget_set_sensitive(self->button_rec, FALSE);
-		gtk_widget_set_sensitive(self->button_stop, FALSE);
+		gtk_image_set_from_pixbuf (GTK_IMAGE(self->gps_status), 
+				           self->gps_off_icon);
+		self->pause_button_visible = FALSE;
+		self->rec_button_enabled = FALSE;
+		self->stop_button_enabled = FALSE;
 		break;
 	case APP_STATE_IDLE:
 	default:
-		status_icon = gtk_icon_theme_load_icon (icon_theme, "gps_not_connected", 48, 0, NULL);
-		button_rec_icon = gtk_icon_theme_load_icon (icon_theme, "camera_video_recording", 48, 0, NULL);
-		update_status_icon = TRUE;
-		update_button_image = TRUE;
-		gtk_widget_set_sensitive(self->button_stop, FALSE);
-		gtk_widget_set_sensitive(self->button_rec, TRUE);
+		gtk_image_set_from_pixbuf (GTK_IMAGE(self->gps_status), 
+				           self->gps_off_icon);
+		self->pause_button_visible = FALSE;
+		self->rec_button_enabled = TRUE;
+		self->stop_button_enabled = FALSE;
 #ifndef __arm__
-		g_print(" vagy idle vagy default volt\n");
+		g_print(" idle or default\n");
 #endif
+		break;
     }
-    if (update_status_icon) {
-        if (status_icon == NULL)
-            status_icon = gtk_icon_theme_load_icon (icon_theme,
-	                                     "qgn_list_gene_default_app",
-	                                     48, 0, NULL);
-        gtk_image_set_from_pixbuf (GTK_IMAGE(self->gps_status), status_icon);
-        gtk_widget_show_all (self->gps_status);
-    }
-    if (update_button_image) {
-        if (button_rec_icon == NULL)
-            button_rec_icon = gtk_icon_theme_load_icon (icon_theme,
-	                                     "qgn_list_gene_default_app",
-	                                     48, 0, NULL);
-        button_rec_image = gtk_image_new_from_pixbuf (button_rec_icon);
-        gtk_misc_set_padding (GTK_MISC (button_rec_image), PADDING, PADDING);
-        g_object_unref (G_OBJECT (button_rec_icon));
-        gtk_button_set_image (GTK_BUTTON (self->button_rec), button_rec_image);
-        gtk_widget_show_all (self->button_rec);
-    }
+    gtk_widget_queue_draw(GTK_WIDGET(self));
 }
 
-void
+static void
 handle_button_press (GtkWidget *button,
 		     gpointer data)
 {
@@ -440,7 +522,10 @@ handle_button_press (GtkWidget *button,
             self->app_state = APP_STATE_STARTING;
 #ifndef __arm__
 	    g_print(" new state: starting\n");
+            g_print("handle_button_press : calling update_status_and_buttons\n");
 #endif
+            update_status_and_buttons(self);
+	    location_gps_device_reset_last_known(self->device);
             location_gpsd_control_start(self->control);
 	    /* open a new log file */
 	    logfile_start(self);
@@ -449,69 +534,149 @@ handle_button_press (GtkWidget *button,
             self->app_state = APP_STATE_PAUSED;
 #ifndef __arm__
 	    g_print(" new state: paused\n");
+            g_print("handle_button_press : calling update_status_and_buttons\n");
 #endif
+            update_status_and_buttons(self);
             location_gpsd_control_stop(self->control);
 	}
 	else if (self->app_state == APP_STATE_PAUSED) {
             self->app_state = APP_STATE_STARTING;
 #ifndef __arm__
 	    g_print(" new state: starting, after pause\n");
+            g_print("handle_button_press : calling update_status_and_buttons\n");
 #endif
+            update_status_and_buttons(self);
 	    /* close tarck segment and start a new one,
 	     * if there were points recorded in the previous
 	     * segment */
 	    if (self->points_recorded_in_current_segment)
 	        g_fprintf(self->file, "</trkseg>\n<trkseg>\n");
+	    location_gps_device_reset_last_known(self->device);
             location_gpsd_control_start(self->control);
 	}
     } else if (button == self->button_stop) {
 	self->app_state = APP_STATE_STOPPING;
 #ifndef __arm__
 	g_print(" new state: stoping\n");
+        g_print("handle_button_press : calling update_status_and_buttons\n");
 #endif
+        update_status_and_buttons(self);
         location_gpsd_control_stop(self->control);
 	/* close log file */
 	logfile_close(self);
     }
-    
-    update_status_and_buttons(self);
 }
 
-
-static void on_error(LocationGPSDControl *control, gpointer data)
+static gboolean
+on_button_release(GtkWidget* widget, GdkEventButton *button_event, gpointer data)
 {
     GpsDataLoggerWidget *self = GPS_DATA_LOGGER_WIDGET (data);
+
+    self->rec_button_down = FALSE;
+    self->stop_button_down = FALSE;
+
+    if (button_event->x >= self->button_rec->allocation.x &&
+        button_event->y >= self->button_rec->allocation.y &&
+        button_event->x <= self->button_rec->allocation.x +
+                       self->button_rec->allocation.width &&
+        button_event->y <= self->button_rec->allocation.y +
+                       self->button_rec->allocation.height &&
+        self->rec_button_enabled) {
+            handle_button_press(self->button_rec, self);
+    } else if (button_event->x >= self->button_stop->allocation.x &&
+        button_event->y >= self->button_stop->allocation.y &&
+        button_event->x <= self->button_stop->allocation.x +
+                       self->button_stop->allocation.width &&
+        button_event->y <= self->button_stop->allocation.y +
+                       self->button_stop->allocation.height &&
+        self->stop_button_enabled) {
+            handle_button_press(self->button_stop, self);
+    }	
+    
+    return FALSE;
+}
+
+static gboolean
+on_button_press(GtkWidget* widget, GdkEventButton *button_event, gpointer data)
+{
+    GpsDataLoggerWidget *self = GPS_DATA_LOGGER_WIDGET (data);
+
+    if (button_event->x >= self->button_rec->allocation.x &&
+        button_event->y >= self->button_rec->allocation.y &&
+        button_event->x <= self->button_rec->allocation.x +
+                       self->button_rec->allocation.width &&
+        button_event->y <= self->button_rec->allocation.y +
+                       self->button_rec->allocation.height &&
+        self->rec_button_enabled)
+    {
+        self->rec_button_down = TRUE;
+        gtk_widget_queue_draw(widget);
+    } else if (button_event->x >= self->button_stop->allocation.x &&
+        button_event->y >= self->button_stop->allocation.y &&
+        button_event->x <= self->button_stop->allocation.x +
+                       self->button_stop->allocation.width &&
+        button_event->y <= self->button_stop->allocation.y +
+                       self->button_stop->allocation.height &&
+        self->stop_button_enabled)
+    {
+        self->stop_button_down = TRUE;
+        gtk_widget_queue_draw(widget);
+    }
+
+    return FALSE;
+
+}
+
+static void
+on_error(LocationGPSDControl *control, gpointer data)
+{
+    GpsDataLoggerWidget *self = GPS_DATA_LOGGER_WIDGET (data);
+#ifndef __arm__
     g_printf("%s\n", __func__);
+#endif
     if (self->app_state != APP_STATE_IDLE) {
 	hildon_banner_show_information(NULL,
 			NULL,
 			"on_error, going to IDLE state");
 	self->app_state = APP_STATE_IDLE;
 	location_gpsd_control_stop(self->control);
+#ifndef __arm__
+        g_print("on_error : calling update_status_and_buttons\n");
+#endif
 	update_status_and_buttons(self);
 	logfile_close(self);
     }
 }
 
-static void on_start(LocationGPSDControl *control, gpointer data)
+static void
+on_start(LocationGPSDControl *control, gpointer data)
 {
+#ifndef __arm__
         g_printf("%s\n", __func__);
+#endif
 }
 
-static void on_stop(LocationGPSDControl *control, gpointer data)
+static void
+on_stop(LocationGPSDControl *control, gpointer data)
 {
 	GpsDataLoggerWidget *self = GPS_DATA_LOGGER_WIDGET (data);
+#ifndef __arm__
         g_printf("%s\n", __func__);
+#endif
 	if (self->app_state == APP_STATE_STOPPING) {
         	hildon_banner_show_information(NULL,
 			                       NULL,
 		        	               "on_stop, going to IDLE state");
 		self->app_state = APP_STATE_IDLE;
+#ifndef __arm__
+                g_print("on_stop : calling update_status_and_buttons\n");
+#endif
 		update_status_and_buttons(self);
 	}
 }
 
-static void on_changed(LocationGPSDevice *device, gpointer data)
+static void
+on_changed(LocationGPSDevice *device, gpointer data)
 {
 	GpsDataLoggerWidget *self = GPS_DATA_LOGGER_WIDGET (data);
 	time_t tme;
@@ -533,6 +698,9 @@ static void on_changed(LocationGPSDevice *device, gpointer data)
                                 "on_changed, going to RECORDING state");
                   
                             self->app_state = APP_STATE_RECORDING;
+#ifndef __arm__
+                            g_print("on_change : calling update_status_and_buttons\n");
+#endif
                             update_status_and_buttons(self);
 	                    self->points_recorded_in_current_segment = 0;
 			}
@@ -633,95 +801,229 @@ static void on_changed(LocationGPSDevice *device, gpointer data)
         }
 }
 
-
-GtkWidget *
-custom_button_new (int padding, char * icon_name)
+static void
+draw_pixbuf(cairo_t *cr, GdkPixbuf *pbuf, int x, int y, gboolean dimmed)
 {
-    GtkIconTheme *icon_theme;
-    GdkPixbuf *icon;
-    GtkWidget *icon_image, *button;
+        int pbuf_width, pbuf_height;
 
-    icon_theme = gtk_icon_theme_get_default ();
-    icon = gtk_icon_theme_load_icon (icon_theme, icon_name, 48, 0, NULL);
-    if (icon == NULL)
-	icon = gtk_icon_theme_load_icon (icon_theme,
-					 "qgn_list_gene_default_app",
-					 48, 0, NULL);
+        pbuf_width = gdk_pixbuf_get_width(pbuf);
+        pbuf_height = gdk_pixbuf_get_height(pbuf);
 
-    icon_image = gtk_image_new_from_pixbuf (icon);
-    gtk_misc_set_padding (GTK_MISC (icon_image), padding, padding);
-    g_object_unref (G_OBJECT (icon));
-    button = gtk_button_new ();
-    gtk_container_add (GTK_CONTAINER (button), icon_image);
-
-    gtk_widget_show_all (button);
-
-    return button;
+        gdk_cairo_set_source_pixbuf(cr, pbuf, 
+			            x - (pbuf_width / 2),
+                                    y - (pbuf_height / 2));
+	if (dimmed)
+	    cairo_paint_with_alpha(cr, 0.3);
+	else
+            cairo_paint(cr);
 }
 
-GtkWidget *
-custom_image_new (char * icon_name)
+static void
+draw_rounded_rectangle(cairo_t *cr, double x, double y,
+                double width, double height, double rad)
 {
-    GtkIconTheme *icon_theme;
-    GdkPixbuf *icon;
-    GtkWidget *icon_image;
+        cairo_move_to(cr, x+rad, y);
+        cairo_line_to(cr, x+width-rad, y);
+        cairo_curve_to(cr, x+width, y, x+width, y, x+width, y+rad);
+        cairo_line_to(cr, x+width, y+height-rad);
+        cairo_curve_to(cr, x+width, y+height, x+width, y+height, x+width-rad, y+height);
+        cairo_line_to(cr, x+rad, y+height);
+        cairo_curve_to(cr, x, y+height, x, y+height, x, y+height-rad);
+        cairo_line_to(cr, x, y+rad);
+        cairo_curve_to(cr, x, y, x, y, x+rad, y);
+        cairo_close_path(cr);
+}
 
-    icon_theme = gtk_icon_theme_get_default ();
-    icon = gtk_icon_theme_load_icon (icon_theme, icon_name, 48, 0, NULL);
-    if (icon == NULL)
-	icon = gtk_icon_theme_load_icon (icon_theme,
-					 "qgn_list_gene_default_app",
-					 48, 0, NULL);
+static void
+draw_background(cairo_t *cr, GdkRectangle *area, gint border, gdouble transparency)
+{
+        cairo_pattern_t *grad;
 
-    icon_image = gtk_image_new_from_pixbuf (icon);
+	gdk_cairo_rectangle(cr, area);
+        cairo_clip(cr);
 
-    return icon_image;
+        cairo_set_operator(cr, CAIRO_OPERATOR_OVER);
+
+        /* Gradient background */
+        grad = cairo_pattern_create_linear(area->x+border,
+                        area->y+border, area->x+border,
+                        area->y+area->height-(2*border));
+        cairo_pattern_add_color_stop_rgba(grad, 0, 0.4, 0.4, 0.4, transparency);
+        cairo_pattern_add_color_stop_rgba(grad, 1, 0.05, 0.05, 0.05, transparency);
+
+        /* Draw border */
+        cairo_set_line_width(cr, 2);
+        draw_rounded_rectangle(cr, area->x+border,
+                        area->y+border, area->width-(2*border),
+                        area->height-(2*border), 10.0);
+        cairo_set_source_rgba(cr, 0.4, 0.4, 0.4, transparency);
+        cairo_stroke_preserve(cr);
+
+        /* Fill background with gradient */
+        cairo_set_source(cr, grad);
+        cairo_fill(cr);
+
+        cairo_pattern_destroy(grad);
+}
+
+
+static gboolean
+on_background_expose(GtkWidget *widget, GdkEventExpose *expose_event, gpointer data)
+{
+    /* GpsDataLoggerWidget *self = GPS_DATA_LOGGER_WIDGET (data); */
+    cairo_t *cr = NULL;
+
+    cr = gdk_cairo_create(widget->window);
+
+    if (!cr) 
+        return FALSE;
+
+    cairo_set_operator(cr, CAIRO_OPERATOR_CLEAR);
+    gdk_cairo_region(cr, expose_event->region);
+    cairo_fill(cr);
+
+    draw_background(cr, &expose_event->area, 3, 0.6);
+
+    cairo_destroy(cr);
+
+    return FALSE;
+}
+
+static gboolean
+on_button_expose(GtkWidget *widget, GdkEventExpose *expose_event, gpointer data)
+{
+    GpsDataLoggerWidget *self = GPS_DATA_LOGGER_WIDGET (data);
+
+    cairo_t *cr = NULL;
+    cr = gdk_cairo_create(widget->window);
+
+    if (!cr) return FALSE;
+
+    /* button background */
+    /* If theme images are not loaded draw with cairo */
+    if (!theme_pixbufs_loaded(self))
+    {
+	if (((widget == self->button_rec) && (self->rec_button_down)) ||
+	   ((widget == self->button_stop) && (self->stop_button_down)))
+                draw_background(cr, &expose_event->area, 0, 0.6);
+            else
+                draw_background(cr, &expose_event->area, 0, 0.8);
+    } else {
+        if (((widget == self->button_rec) && (self->rec_button_down)) ||
+            ((widget == self->button_stop) && (self->stop_button_down))) 
+                draw_pixbuf(cr, self->button_pressed_bg,
+			    widget->allocation.x + widget->allocation.width/2,
+			    widget->allocation.y + widget->allocation.height/2,
+			    FALSE);
+        else if (((widget == self->button_rec) && (! self->rec_button_enabled)) ||
+                ((widget == self->button_stop) && (! self->stop_button_enabled))) 
+                draw_pixbuf(cr, self->button_bg,
+			    widget->allocation.x + widget->allocation.width/2,
+			    widget->allocation.y + widget->allocation.height/2,
+			    TRUE);
+        else
+                draw_pixbuf(cr, self->button_bg,
+			    widget->allocation.x + widget->allocation.width/2,
+			    widget->allocation.y + widget->allocation.height/2,
+			    FALSE);
+    }
+    /* button icons */
+    if (widget == self->button_stop)
+            draw_pixbuf(cr, self->stop_icon,
+			    widget->allocation.x + widget->allocation.width/2,
+			    widget->allocation.y + widget->allocation.height/2,
+			    ! self->stop_button_enabled);
+    else if ((widget == self->button_rec) && (self->pause_button_visible))
+            draw_pixbuf(cr, self->pause_icon,
+			    widget->allocation.x + widget->allocation.width/2,
+			    widget->allocation.y + widget->allocation.height/2,
+			    ! self->rec_button_enabled);
+    else if ((widget == self->button_rec) && (! self->pause_button_visible))
+            draw_pixbuf(cr, self->rec_icon,
+			    widget->allocation.x + widget->allocation.width/2,
+			    widget->allocation.y + widget->allocation.height/2,
+			    ! self->rec_button_enabled);
+
+    cairo_destroy(cr);
+
+    return FALSE;
 }
 
 static void
 gps_data_logger_widget_init (GpsDataLoggerWidget * self)
 {
     GtkWidget *vbox, *hbox_status, *hbox_buttons;
-    GtkWidget *button_rec, *button_stop, *status_icon, *status_text;
+    GtkWidget *status_text;
+    GdkScreen *screen;
+    GdkColormap *colormap;
 
     self->file = NULL;
+    self->app_state = APP_STATE_IDLE;
+    /* Preload all icon pixmaps */
+    load_icons(self);
+    load_theme_pixbufs(self);
+
+    /* Needed for transparency */
+    screen = gtk_widget_get_screen(GTK_WIDGET(self));
+    colormap = gdk_screen_get_rgba_colormap(screen);
+    gtk_widget_set_colormap(GTK_WIDGET(self), colormap);
+
+    self->background = gtk_event_box_new();
+    gtk_widget_set_app_paintable(self->background, TRUE);
+    gtk_container_add(GTK_CONTAINER(self), self->background);
+
+    vbox = gtk_vbox_new (FALSE, HILDON_MARGIN_HALF);
+    gtk_container_set_border_width(GTK_CONTAINER(vbox), HILDON_MARGIN_DEFAULT);
+    gtk_container_add(GTK_CONTAINER(self->background), vbox);
+
+    self->rec_button_down = FALSE;
+    self->rec_button_enabled = TRUE;
+    self->button_rec = gtk_event_box_new();
+    if (theme_pixbufs_loaded(self))
+        gtk_widget_set_size_request(self->button_rec, 
+		        gdk_pixbuf_get_width(self->button_bg),
+                        gdk_pixbuf_get_height(self->button_bg));
+    else
+        gtk_widget_set_size_request(self->button_rec,96,96); 
+    gtk_widget_set_app_paintable(self->button_rec, TRUE);
+    gtk_event_box_set_visible_window(GTK_EVENT_BOX(self->button_rec), FALSE);
     
-    button_rec = gtk_button_new();
-    button_stop = custom_button_new (PADDING, "camera_video_stop");
-    status_icon = gtk_image_new();
-    
+    self->stop_button_down = FALSE;
+    self->stop_button_enabled = FALSE;
+    self->button_stop = gtk_event_box_new();
+    if (theme_pixbufs_loaded(self))
+        gtk_widget_set_size_request(self->button_stop,
+		        gdk_pixbuf_get_width(self->button_bg),
+                        gdk_pixbuf_get_height(self->button_bg));
+    else
+        gtk_widget_set_size_request(self->button_stop,96,96);
+    gtk_widget_set_app_paintable(self->button_stop, TRUE);
+    gtk_event_box_set_visible_window(GTK_EVENT_BOX(self->button_stop), FALSE);
+
+    self->gps_status = gtk_image_new();    
     status_text = gtk_label_new("Status: ");
 
-    vbox = gtk_vbox_new (FALSE, 0);
     hbox_status = gtk_hbox_new (FALSE, 2);
     hbox_buttons = gtk_hbox_new (FALSE, 2);
     gtk_box_pack_start (GTK_BOX (hbox_status), status_text, FALSE, TRUE, 0);
-    gtk_box_pack_start (GTK_BOX (hbox_status), status_icon, FALSE, FALSE, 0);
-    gtk_box_pack_start (GTK_BOX (hbox_buttons), button_rec, FALSE, FALSE, 0);
-    gtk_box_pack_start (GTK_BOX (hbox_buttons), button_stop, FALSE, FALSE, 0);
+    gtk_box_pack_start (GTK_BOX (hbox_status), self->gps_status, FALSE, FALSE, 0);
+    gtk_box_pack_start (GTK_BOX (hbox_buttons), self->button_rec, FALSE, FALSE, 0);
+    gtk_box_pack_start (GTK_BOX (hbox_buttons), self->button_stop, FALSE, FALSE, 0);
     gtk_box_pack_start (GTK_BOX (vbox), hbox_status, FALSE, FALSE, 0);
     gtk_box_pack_start (GTK_BOX (vbox), hbox_buttons, FALSE, FALSE, 0);
-    
-    self->gps_status = status_icon;
-    self->button_rec = button_rec;
-    self->button_stop = button_stop;
-    self->app_state = APP_STATE_IDLE;
-    
+     
+#ifndef __arm__
+    g_print("init : calling update_status_and_buttons\n");
+#endif
     update_status_and_buttons(self);
+
+    gtk_widget_show_all (self->background);
     
-    g_signal_connect (self->button_rec, "clicked",
-		      G_CALLBACK (handle_button_press), self);
-    g_signal_connect (self->button_stop, "clicked",
-		      G_CALLBACK (handle_button_press), self);
-
-    gtk_widget_show_all (vbox);
-
     /* Set the resizing behavior */
     /*
     hd_home_widget_item_set_resize_type (HD_HOME_PLUGIN_ITEM (home_widget),
 					 HD_HOME_PLUGIN_ITEM_RESIZE_BOTH);
     */
-    gtk_container_add (GTK_CONTAINER (self), vbox);
 
     self->control = location_gpsd_control_get_default();
 
@@ -729,7 +1031,8 @@ gps_data_logger_widget_init (GpsDataLoggerWidget * self)
     apply_settings(self);
 
     self->device  = g_object_new(LOCATION_TYPE_GPS_DEVICE, NULL);
-
+    
+    /* location signals */
     g_signal_connect(self->control, "error", G_CALLBACK(on_error), self);
     g_signal_connect(self->control, "gpsd-running", G_CALLBACK(on_start),
                                                                    self);
@@ -740,9 +1043,29 @@ gps_data_logger_widget_init (GpsDataLoggerWidget * self)
 
     /* widget settings */
     hd_home_plugin_item_set_settings (HD_HOME_PLUGIN_ITEM (self), TRUE);
-    g_signal_connect (self, "show-settings", G_CALLBACK (settings_dialog_show),
+    g_signal_connect (self, "show-settings", G_CALLBACK (on_settings_dialog_show),
 		    self);
 
+    /* button signals */
+
+    g_signal_connect(G_OBJECT(self->background), "button-release-event",
+                    G_CALLBACK(on_button_release), self);
+    g_signal_connect(G_OBJECT(self->background), "button-press-event",
+                    G_CALLBACK(on_button_press), self);
+    g_signal_connect(G_OBJECT(self->background), "leave-notify-event",
+                    G_CALLBACK(on_widget_leave), self);
+    /* expose events */
+    g_signal_connect(self->background, "expose-event", 
+		     G_CALLBACK(on_background_expose),
+		     (gpointer)self);
+    g_signal_connect(self->button_rec, "expose-event", 
+                     G_CALLBACK(on_button_expose),
+                     (gpointer)self);
+    g_signal_connect(self->button_stop, "expose-event", 
+                     G_CALLBACK(on_button_expose),
+                     (gpointer)self);
+    g_signal_connect(G_OBJECT(self->background), "style-set",
+                     G_CALLBACK(on_style_set), (gpointer)self);
 }
 
 static void
@@ -757,6 +1080,8 @@ gps_data_logger_widget_finalize (GObject *object)
     }
     settings_free(self->settings);
     g_free(self->settings);
+    free_icons(self);
+    free_theme_pixbufs(self);
 		    
     /* Call the base class's implementation: */
     G_OBJECT_CLASS(gps_data_logger_widget_parent_class)->finalize (object);
